Re-exports
pub use ops::Op;pub use ops::OperationMutator;pub use ops::activation_ops;pub use ops::all_ops;pub use ops::math_ops;pub use regression::Accuracy;pub use regression::AccuracyResult;pub use regression::DataSet;pub use regression::Loss;pub use regression::Regression;pub use collections::*;
Modules
Enums
- Arity
- Arity is a way to describe how many inputs an operation expects. It can be zero, a specific number, or any number.