GraphEvalCache

Struct GraphEvalCache 

pub struct GraphEvalCache<V> { /* private fields */ }
A cache for storing intermediate results during graph evaluation.

This cache is used to store the inputs and outputs of each node in the graph during evaluation, allowing for more efficient re-evaluation of nodes when their inputs change. If we want to save a graph’s evaluation between different evals, we need to keep track of the inputs and outputs from previous runs incase of recurrent structures. This cache is the answer to that.