Enum Arity

pub enum Arity {
    Zero,
    Exact(usize),
    Any,
}
Arity is a way to describe how many inputs an operation expects. It can be zero, a specific number, or any number.

Variants



Zero



Exact(usize)



Any

Methods from Deref<Target = usize>

1.43.0 · Source

pub const MIN: usize = 0usize

1.43.0 · Source

pub const MAX: usize = 18_446_744_073_709_551_615usize

1.53.0 · Source

pub const BITS: u32 = 64u32