NodeExt

Trait NodeExt 

pub trait NodeExt: Node {
    // Provided method
    fn set_value(&mut self, value: Self::Value) { ... }
}
NodeExt is an extension trait for the Node trait that provides additional functionality for working with nodes. Super stupid little thing but I got tired of writing out the same code over and over again.

Provided Methods

fn set_value(&mut self, value: Self::Value)

Implementors

Source

impl<N: Node> NodeExt for N